# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from platform import platform
import pylink.library as library
import pylink.util as util

import mock

import unittest


class TestLibrary(unittest.TestCase):
    """Unit test for the ``library`` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance

        Returns:
          ``None``
        """
        assertRaisesRegexp = getattr(self, 'assertRaisesRegexp', None)
        self.assertRaisesRegexp = getattr(self, 'assertRaisesRegex', assertRaisesRegexp)
        self.lib_path = '/'

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance

        Returns:
          ``None``
        """
        pass

    def mock_directories(self, mock_os, structure, sep):
        """Mocks a directory structure.

        This function is used to mock a directory structure, so that checking
        if something is a file, is a directory, or listing directories will use
        our mocked structure instead.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): the mocked ``os`` module
          structure (list): a list of directories or files
          sep (str): the operating system seperator

        Returns:
          ``None``
        """
        def isfile(f):
            """Returns whether the file exists in the structure."""
            if not any(s.endswith(f) for s in structure):
                return False
            return '.' in f.split(sep)[-1]

        mock_os.path.isfile.side_effect = isfile

        def isdir(f):
            """Returns whether the directory exists within the structure."""
            if not any(s.startswith(f) for s in structure):
                return False
            return not isfile(f)

        mock_os.path.isdir.side_effect = isdir

        def join(*args):
            """Joins several strings to form a path."""
            s = ''
            for arg in args:
                if not s.endswith(sep) and len(s) > 0:
                    s += sep
                s += arg
            return s

        mock_os.path.join.side_effect = join

        def listdir(f):
            """List the files and directories within the directory."""
            if not isdir(f):
                return list()

            directories = []
            for s in structure:
                if s.startswith(f):
                    s = s[len(f):]
                    if s.startswith(sep):
                        s = s[len(sep):]
                    directories.append(s.split(sep)[0])
            return directories

        mock_os.listdir.side_effect = listdir

        def mock_walk(b):
            r = list()
            dirname = b
            if not isdir(dirname):
                return []

            subdirs = filter(len, list(set([d for d in listdir(dirname) if isdir(join(dirname, d))])))
            subfiles = filter(len, list(set([f for f in listdir(dirname) if isfile(join(dirname, f))])))
            r.append((dirname, subdirs, subfiles))
            for subdir in subdirs:
                r.extend(mock_walk(join(dirname, subdir)))

            return r

        mock_os.walk.return_value = mock_walk(sep)

    @mock.patch('sys.platform', new='darwin')
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    def test_initialize_default(self, mock_load_library, mock_find_library, mock_open):
        """Tests creating a library and finding the default DLL.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): mock for mocking the
            ``ctypes.util.find_library()`` call
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_find_library.return_value = self.lib_path

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.JLINK_SDK_OBJECT)
        mock_open.assert_called_with(self.lib_path, 'rb')
        mock_load_library.assert_called_once()

    @mock.patch('sys.platform', new='darwin')
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    @mock.patch('os.path.isdir')
    def test_initialiaze_no(self, mock_isdir, mock_load_library, mock_find_library, mock_open):
        """Tests creating a library when the default DLL does not exist.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_isdir (Mock): mock for mocking the call to ``os.path.isdir``
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): mock for mocking the
            ``ctypes.util.find_library()`` call
          mock_open (Mock): mock for mocking the clal to ``open()``

        Returns:
          ``None``
        """
        mock_isdir.return_value = False
        mock_find_library.return_value = None

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.JLINK_SDK_OBJECT)
        self.assertEqual(1, mock_find_library.call_count)
        self.assertEqual(0, mock_load_library.call_count)

    @mock.patch('sys.platform', new='darwin')
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    def test_initialize_with_path(self, mock_load_library, mock_find_library, mock_open):
        """Tests creating a library when passing in a DLL path.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): mock for mocking the
            ``ctypes.util.find_library()`` call
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_find_library.return_value = None

        lib = library.Library(self.lib_path)
        lib.unload = mock.Mock()

        self.assertEqual(0, mock_find_library.call_count)

        mock_open.assert_called_with(self.lib_path, 'rb')
        mock_load_library.assert_called_once()

    @mock.patch('sys.platform', new='windows')
    @mock.patch('sys.maxsize', new=(2**63-1))
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('pylink.library.ctypes')
    def test_initialize_windows(self, mock_ctypes, mock_find_library, mock_open):
        """Tests creating a library on a Windows machine.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_ctypes (Mock): a mocked version of the ctypes library
          mock_find_library (Mock): mock for mocking the
            ``ctypes.util.find_library()`` call
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_windll = mock.Mock()
        mock_windll.__getitem__ = mock.Mock()

        mock_cdll = mock.Mock()
        mock_cdll.__getitem__ = mock.Mock()

        mock_ctypes.windll = mock_windll
        mock_ctypes.cdll = mock_cdll
        mock_find_library.return_value = self.lib_path

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.WINDOWS_64_JLINK_SDK_NAME)
        mock_open.assert_called_with(self.lib_path, 'rb')
        mock_cdll.LoadLibrary.assert_called_once()
        mock_windll.LoadLibrary.assert_called_once()

    @mock.patch('sys.platform', new='windows')
    @mock.patch('sys.maxsize', new=(2**31-1))
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('pylink.library.ctypes')
    def test_initialize_windows_32bit(self, mock_ctypes, mock_find_library, mock_open):
        """Tests creating a library on a Windows machine with 32bit Python.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_ctypes (Mock): a mocked version of the ctypes library
          mock_find_library (Mock): mock for mocking the
            ``ctypes.util.find_library()`` call
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_windll = mock.Mock()
        mock_windll.__getitem__ = mock.Mock()

        mock_cdll = mock.Mock()
        mock_cdll.__getitem__ = mock.Mock()

        mock_ctypes.windll = mock_windll
        mock_ctypes.cdll = mock_cdll
        mock_find_library.return_value = self.lib_path

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.WINDOWS_32_JLINK_SDK_NAME)
        mock_open.assert_called_with(self.lib_path, 'rb')
        mock_cdll.LoadLibrary.assert_called_once()
        mock_windll.LoadLibrary.assert_called_once()

    @mock.patch('sys.platform', new='darwin')
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    def test_load(self, mock_load_library, mock_find_library, mock_open):
        """Tests that we can pass in a path to a DLL to load.

        If the path is valid, loads the given ``DLL``, otherwise stays the
        same.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): mock for mocking the
            ``ctypes.util.find_library()`` call
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_find_library.return_value = self.lib_path

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.JLINK_SDK_OBJECT)
        self.assertEqual(1, mock_find_library.call_count)

        mock_open.assert_called_with(self.lib_path, 'rb')
        self.assertEqual(1, mock_load_library.call_count)

        new_path = '\\'
        lib.load(new_path)

        mock_open.assert_called_with(new_path, 'rb')
        self.assertEqual(2, mock_load_library.call_count)

        lib.load(None)
        mock_open.assert_called_with(new_path, 'rb')
        self.assertEqual(3, mock_load_library.call_count)

    @mock.patch('sys.platform', new='darwin')
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('pylink.library.open', new=mock.MagicMock())
    @mock.patch('pylink.library.ctypes')
    @mock.patch('os.remove')
    def test_unload_no_library(self, mock_remove, mock_ctypes):
        """Tests unloading the library when no DLL is loaded.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_remove (Mock): the mocked call to ``os.remove()``
          mock_ctypes (Mock): mocked ``ctypes`` module

        Returns:
          ``None``
        """
        lib = library.Library('')
        setattr(lib, '_lib', None)
        setattr(lib, '_temp', None)

        self.assertFalse(lib.unload())

        mock_remove.assert_not_called()

    @mock.patch('sys.platform', new='windows')
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('pylink.library.open', new=mock.MagicMock())
    @mock.patch('pylink.library.ctypes')
    @mock.patch('os.remove')
    def test_unload_windows(self, mock_remove, mock_ctypes):
        """Tests unloading the library on Windows.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_remove (Mock): the mocked call to ``os.remove()``
          mock_ctypes (Mock): mocked ``ctypes`` module

        Returns:
          ``None``
        """
        lib = library.Library('')

        self.assertTrue(lib.unload())

        self.assertEqual(2, mock_ctypes.windll.kernel32.FreeLibrary.call_count)

        mock_remove.assert_called_once()

    @mock.patch('sys.platform', new='darwin')
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('pylink.library.open', new=mock.MagicMock())
    @mock.patch('pylink.library.ctypes')
    @mock.patch('os.remove')
    def test_unload_darwin_linux(self, mock_remove, mock_ctypes):
        """Tests unloading the library on Darwin and Linux platforms.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_remove (Mock): the mocked call to ``os.remove()``
          mock_ctypes (Mock): mocked ``ctypes`` module

        Returns:
          ``None``
        """
        lib = library.Library('')

        self.assertTrue(lib.unload())

        mock_remove.assert_called_once()

        self.assertEqual(None, lib._lib)
        self.assertEqual(None, lib._temp)

    @mock.patch('sys.platform', new='darwin')
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    def test_dll_getter(self, mock_load_library, mock_find_library, mock_open):
        """Tests that the ``.dll()`` getter returns the set ``DLL``.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): mock for mocking the
            ``ctypes.util.find_library()`` call
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_find_library.return_value = self.lib_path
        mock_load_library.return_value = 0xDEADBEEF

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.JLINK_SDK_OBJECT)
        self.assertEqual(1, mock_find_library.call_count)

        mock_open.assert_called_with(self.lib_path, 'rb')
        mock_load_library.assert_called_once()

        self.assertEqual(0xDEADBEEF, lib.dll())

    @mock.patch('sys.platform', new='darwin')
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    @mock.patch('pylink.library.os')
    def test_darwin_4_98_e(self, mock_os, mock_load_library, mock_find_library, mock_open):
        """Tests finding the DLL on Darwin through the SEGGER application for V4.98E-.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_find_library.return_value = None
        directories = [
            '/Applications/SEGGER/JLink 1/libjlinkarm.dylib'
        ]

        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.JLINK_SDK_OBJECT)
        self.assertEqual(1, mock_find_library.call_count)
        self.assertEqual(1, mock_load_library.call_count)

    @mock.patch('sys.platform', new='darwin')
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    @mock.patch('pylink.library.os')
    def test_darwin_5_0_0(self, mock_os, mock_load_library, mock_find_library, mock_open):
        """Tests finding the DLL on Darwin through the SEGGER application for V5.0.0+.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_find_library.return_value = None
        directories = [
            '/Applications/SEGGER/JLink/libjlinkarm.5.12.10.dylib',
            '/Applications/SEGGER/JLink/libjlinkarm.5.dylib'
        ]

        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.JLINK_SDK_OBJECT)
        self.assertEqual(1, mock_find_library.call_count)
        self.assertEqual(1, mock_load_library.call_count)

    @mock.patch('sys.platform', new='darwin')
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    @mock.patch('pylink.library.os')
    def test_darwin_6_0_0(self, mock_os, mock_load_library, mock_find_library, mock_open):
        """Tests finding the DLL on Darwin through the SEGGER application for V6.0.0+.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_find_library.return_value = None
        directories = [
            '/Applications/SEGGER/JLink/libjlinkarm.dylib'
        ]

        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.JLINK_SDK_OBJECT)
        self.assertEqual(1, mock_find_library.call_count)
        self.assertEqual(1, mock_load_library.call_count)

    @mock.patch('sys.platform', new='darwin')
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    @mock.patch('pylink.library.os')
    def test_darwin_empty(self, mock_os, mock_load_library, mock_find_library, mock_open):
        """Tests finding the DLL on Darwin through the SEGGER application for V6.0.0+.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_find_library.return_value = None
        directories = [
            '/Applications/SEGGER/JLink/'
        ]

        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.JLINK_SDK_OBJECT)
        self.assertEqual(1, mock_find_library.call_count)
        self.assertEqual(0, mock_load_library.call_count)

    @mock.patch('sys.platform', new='windows')
    @mock.patch('sys.maxsize', new=(2**31 - 1))
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('pylink.library.ctypes')
    @mock.patch('pylink.library.os')
    def test_windows_4_98_e(self, mock_os, mock_ctypes, mock_find_library, mock_open):
        """Tests finding the DLL on Windows through the SEGGER application for V4.98E-.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_ctypes (Mock): a mocked version of the ctypes library
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_windll = mock.Mock()
        mock_windll.__getitem__ = mock.Mock()

        mock_cdll = mock.Mock()
        mock_cdll.__getitem__ = mock.Mock()

        mock_ctypes.windll = mock_windll
        mock_ctypes.cdll = mock_cdll
        mock_find_library.return_value = None

        directories = [
            'C:\\Program Files\\SEGGER\\JLink_V49e\\JLinkARM.dll'
        ]

        self.mock_directories(mock_os, directories, '\\')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.WINDOWS_32_JLINK_SDK_NAME)
        self.assertEqual(1, mock_find_library.call_count)
        self.assertEqual(1, mock_windll.LoadLibrary.call_count)
        self.assertEqual(1, mock_cdll.LoadLibrary.call_count)

    @mock.patch('sys.platform', new='windows')
    @mock.patch('sys.maxsize', new=(2**31-1))
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('pylink.library.ctypes')
    @mock.patch('pylink.library.os')
    def test_windows_5_10_0(self, mock_os, mock_ctypes, mock_find_library, mock_open):
        """Tests finding the DLL on Windows through the SEGGER application for V5.0.0+.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_ctypes (Mock): a mocked version of the ctypes library
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_windll = mock.Mock()
        mock_windll.__getitem__ = mock.Mock()

        mock_cdll = mock.Mock()
        mock_cdll.__getitem__ = mock.Mock()

        mock_ctypes.windll = mock_windll
        mock_ctypes.cdll = mock_cdll
        mock_find_library.return_value = None

        directories = [
            'C:\\Program Files (x86)\\SEGGER\\JLink_V510l\\JLinkARM.dll'
        ]

        self.mock_directories(mock_os, directories, '\\')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.WINDOWS_32_JLINK_SDK_NAME)
        self.assertEqual(1, mock_find_library.call_count)
        self.assertEqual(1, mock_windll.LoadLibrary.call_count)
        self.assertEqual(1, mock_cdll.LoadLibrary.call_count)

    @mock.patch('sys.platform', new='windows')
    @mock.patch('sys.maxsize', new=(2**31-1))
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('pylink.library.ctypes')
    @mock.patch('pylink.library.os')
    def test_windows_jlinkarm(self, mock_os, mock_ctypes, mock_find_library, mock_open):
        """Tests finding the DLL on Windows through the SEGGER JLinkARM folder.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_ctypes (Mock): a mocked version of the ctypes library
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_windll = mock.Mock()
        mock_windll.__getitem__ = mock.Mock()

        mock_cdll = mock.Mock()
        mock_cdll.__getitem__ = mock.Mock()

        mock_ctypes.windll = mock_windll
        mock_ctypes.cdll = mock_cdll
        mock_find_library.return_value = None

        directories = [
            'C:\\Program Files (x86)\\SEGGER\\JLinkARM\\JLinkARM.dll'
        ]

        self.mock_directories(mock_os, directories, '\\')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.WINDOWS_32_JLINK_SDK_NAME)
        self.assertEqual(1, mock_find_library.call_count)
        self.assertEqual(1, mock_windll.LoadLibrary.call_count)
        self.assertEqual(1, mock_cdll.LoadLibrary.call_count)

    @mock.patch('sys.platform', new='windows')
    @mock.patch('sys.maxsize', new=(2**31 - 1))
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('pylink.library.ctypes')
    @mock.patch('pylink.library.os')
    def test_windows_empty(self, mock_os, mock_ctypes, mock_find_library, mock_open):
        """Tests finding the DLL on Windows through the SEGGER application for V6.0.0+.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_ctypes (Mock): a mocked version of the ctypes library
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open``

        Returns:
          ``None``
        """
        mock_windll = mock.Mock()
        mock_windll.__getitem__ = mock.Mock()

        mock_cdll = mock.Mock()
        mock_cdll.__getitem__ = mock.Mock()

        mock_ctypes.windll = mock_windll
        mock_ctypes.cdll = mock_cdll
        mock_find_library.return_value = None

        directories = [
            'C:\\Program Files\\',
            'C:\\Program Files (x86)\\'
        ]

        self.mock_directories(mock_os, directories, '\\')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.WINDOWS_32_JLINK_SDK_NAME)
        self.assertEqual(1, mock_find_library.call_count)
        self.assertEqual(0, mock_windll.LoadLibrary.call_count)
        self.assertEqual(0, mock_cdll.LoadLibrary.call_count)

    @mock.patch('sys.platform', new='cygwin')
    @mock.patch('sys.maxsize', new=(2**31-1))
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    @mock.patch('pylink.library.os')
    def test_cygwin(self, mock_os, mock_load_library, mock_find_library, mock_open):
        """Tests finding the DLL when running within Cygwin.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_find_library.return_value = None

        directories = [
            'C:\\Program Files (x86)\\SEGGER\\JLinkARM\\JLinkARM.dll',
            'C:\\Program Files (x86)\\SEGGER\\JLink_V500l\\JLinkARM.dll'
        ]

        self.mock_directories(mock_os, directories, '\\')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.WINDOWS_32_JLINK_SDK_NAME)
        self.assertEqual(1, mock_find_library.call_count)
        self.assertEqual(1, mock_load_library.call_count)

    @mock.patch('sys.platform', new='linux')
    @mock.patch('pylink.util.is_os_64bit', return_value=False)
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    @mock.patch('pylink.library.os')
    def test_linux_4_98_e(self, mock_os, mock_load_library, mock_find_library, mock_open, mock_is_os_64bit):
        """Tests finding the DLL on Linux through the SEGGER application for V4.98E-.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_find_library.return_value = None
        directories = [
            '/opt/SEGGER/JLink_Linux_V498e_i386/libjlinkarm.so',
        ]

        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()
        load_library_args, load_libary_kwargs = mock_load_library.call_args
        self.assertEqual(directories[0], lib._path)

    @mock.patch('sys.platform', new='linux2')
    @mock.patch('pylink.util.is_os_64bit', return_value=False)
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    @mock.patch('pylink.library.os')
    def test_linux_6_10_0_32bit(self, mock_os, mock_load_library, mock_find_library, mock_open, mock_is_os_64bit):
        """Tests finding the DLL on Linux through the SEGGER application for V6.0.0+ on 32 bit linux.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``
          mock_is_os_64bit (Mock): mock for mocking the call to ``is_os_64bit``, returns False

        Returns:
          ``None``
        """
        mock_find_library.return_value = None
        directories = [
            '/opt/SEGGER/JLink_Linux_V610d_x86_64/libjlinkarm_x86.so.6.10',
            '/opt/SEGGER/JLink_Linux_V610d_x86_64/libjlinkarm.so.6.10',
        ]

        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()
        load_library_args, load_libary_kwargs = mock_load_library.call_args
        self.assertEqual(directories[0], lib._path)

        directories = [
            '/opt/SEGGER/JLink_Linux_V610d_x86_64/libjlinkarm.so.6.10',
        ]

        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()
        load_library_args, load_libary_kwargs = mock_load_library.call_args
        self.assertEqual(None, lib._path)

    @mock.patch('sys.platform', new='linux2')
    @mock.patch('pylink.util.is_os_64bit', return_value=True)
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    @mock.patch('ctypes.CDLL')
    @mock.patch('pylink.library.os')
    def test_linux_6_10_0_64bit(self, mock_os, mock_cdll, mock_load_library,
                                mock_find_library, mock_open, mock_is_os_64bit):
        """Tests finding the DLL on Linux through the SEGGER application for V6.0.0+ on 64 bit linux.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_cdll (Mock): a mocked version of the `cdll.CDLL` class constructor
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``
          mock_is_os_64bit (Mock): mock for mocking the call to ``is_os_64bit``, returns True

        Returns:
          ``None``
        """
        mock_find_library.return_value = None
        mock_cdll.return_value = None
        directories = [
            '/opt/SEGGER/JLink_Linux_V610d_x86_64/libjlinkarm_x86.so.6.10',
            '/opt/SEGGER/JLink_Linux_V610d_x86_64/libjlinkarm.so.6.10',
        ]

        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()
        load_library_args, load_libary_kwargs = mock_load_library.call_args
        self.assertEqual(directories[1], lib._path)

        directories = [
            '/opt/SEGGER/JLink_Linux_V610d_x86_64/libjlinkarm_x86.so.6.10',
        ]

        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()
        self.assertEqual(None, lib._path)

    @mock.patch('sys.platform', new='linux2')
    @mock.patch('pylink.util.is_os_64bit', return_value=True)
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    @mock.patch('ctypes.CDLL')
    @mock.patch('pylink.library.os')
    def test_linux_64bit_no_x86(self, mock_os, mock_cdll, mock_load_library,
                                mock_find_library, mock_open, mock_is_os_64bit):
        """Tests finding the DLL on Linux when no library name contains 'x86'.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_cdll (Mock): a mocked version of the `cdll.CDLL` class constructor
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``
          mock_is_os_64bit (Mock): mock for mocking the call to ``is_os_64bit``, returns True

        Returns:
          ``None``
        """
        def on_cdll(name):
            if '_arm' in name:
                raise OSError

        mock_find_library.return_value = None
        mock_cdll.side_effect = on_cdll
        directories = [
            '/opt/SEGGER/JLink_Linux_V610d_x86_64/libjlinkarm_arm.so.6.10',
            '/opt/SEGGER/JLink_Linux_V610d_x86_64/libjlinkarm.so.6.10',
        ]

        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()
        load_library_args, load_libary_kwargs = mock_load_library.call_args
        self.assertEqual(directories[1], lib._path)

    @mock.patch('sys.platform', new='linux')
    @mock.patch('pylink.library.open')
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    @mock.patch('pylink.library.os')
    def test_linux_empty(self, mock_os, mock_load_library, mock_find_library, mock_open):
        """Tests finding the DLL on Linux through the SEGGER application for V6.0.0+.

        Args:
          self (TestLibrary): the ``TestLibrary`` instance
          mock_os (Mock): a mocked version of the ``os`` module
          mock_load_library (Mock): a mocked version of the library loader
          mock_find_library (Mock): a mocked call to ``ctypes`` find library
          mock_open (Mock): mock for mocking the call to ``open()``

        Returns:
          ``None``
        """
        mock_find_library.return_value = None
        directories = []

        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.JLINK_SDK_OBJECT)
        self.assertEqual(1, mock_find_library.call_count)
        self.assertEqual(0, mock_load_library.call_count)

    @mock.patch('os.name', new='posix')
    @mock.patch('sys.platform', new='linux')
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('pylink.library.open')
    @mock.patch('pylink.library.os')
    @mock.patch('pylink.util.is_os_64bit', return_value=True)
    @mock.patch('pylink.platform.libc_ver', return_value=('libc', '1.0'))
    @mock.patch('ctypes.util.find_library', return_value='libjlinkarm.so.7')
    @mock.patch('pylink.library.JLinkarmDlInfo.__init__')
    @mock.patch('ctypes.CDLL')
    @mock.patch('ctypes.cdll.LoadLibrary')
    def test_linux_glibc_unavailable(self, mock_load_library, mock_cdll, mock_dlinfo_ctr, mock_find_library,
                                     mock_libc_ver, mock_is_os_64bit, mock_os,  mock_open):
        """Confirms the whole JLinkarmDlInfo code path is not involved when GNU libc
        extensions are unavailable on a Linux system, and that we'll successfully fallback
        to the "search by file name".

        Test case:
        - initial find_library('jlinkarm') succeeds
        - but the host system does not provide GNU libc extensions
        - we should then skip the dlinfo() dance and proceed
          to the "search by file name" code path, aka find_library_linux()
        - and "successfully load" a mock library file from /opt/SEGGER/JLink
        """
        mock_cdll.side_effect = None
        directories = [
            # Library.find_library_linux() should find this.
            '/opt/SEGGER/JLink/libjlinkarm.so.6'
        ]
        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_called_once_with(library.Library.JLINK_SDK_OBJECT)
        # JLinkarmDlInfo has not been instantiated.
        self.assertEqual(0, mock_dlinfo_ctr.call_count)
        # Fallback to "search by file name" has succeeded.
        self.assertEqual(1, mock_load_library.call_count)
        self.assertEqual(directories[0], lib._path)

    @mock.patch('os.name', new='posix')
    @mock.patch('sys.platform', new='linux')
    @mock.patch('tempfile.NamedTemporaryFile', new=mock.Mock())
    @mock.patch('os.remove', new=mock.Mock())
    @mock.patch('pylink.library.open')
    @mock.patch('pylink.library.os')
    @mock.patch('pylink.util.is_os_64bit', return_value=True)
    @mock.patch('pylink.platform.libc_ver', return_value=('glibc', '2.34'))
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.CDLL')
    @mock.patch('ctypes.cdll.LoadLibrary')
    def test_linux_dl_unavailable(self, mock_load_library, mock_cdll, mock_find_library, mock_libc_ver,
                                  mock_is_os_64bit, mock_os,  mock_open):
        """Confirms we successfully fallback to the "search by file name" code path when libdl is
        unavailable despite the host system presenting itself as POSIX (GNU/Linux).

        Test case:
        - initial find_library('jlinkarm') succeeds
        - the host system presents itself as GNU/Linux, but does not provide libdl
        - we should then skip the dlinfo() dance and proceed
          to the "search by file name" code path, aka find_library_linux()
        - and "successfully load" a mock library file from /opt/SEGGER/JLink
        """
        mock_cdll.side_effect = None
        mock_find_library.side_effect = [
            # find_library('jlinkarm')
            'libjlinkarm.so.6',
            # find_library('dl')
            None
        ]

        directories = [
            '/opt/SEGGER/JLink/libjlinkarm.so.6'
        ]
        self.mock_directories(mock_os, directories, '/')

        lib = library.Library()
        lib.unload = mock.Mock()

        mock_find_library.assert_any_call(library.Library.JLINK_SDK_OBJECT)
        mock_find_library.assert_any_call('dl')
        self.assertEqual(2, mock_find_library.call_count)
        # Called once in JLinkarmDlInfo and once in Library.
        self.assertEqual(2, mock_load_library.call_count)
        # The dlinfo() dance silently failed, but will answer None resolved path.
        self.assertIsNone(library.Library._dlinfo.path)
        # Fallback to "search by file name" has succeeded.
        self.assertEqual(directories[0], lib._path)

    @mock.patch('os.name', new='posix')
    @mock.patch('sys.platform', new='linux')
    @mock.patch('pylink.platform.libc_ver', return_value=('glibc', '2.34'))
    @mock.patch('ctypes.util.find_library')
    @mock.patch('ctypes.cdll.LoadLibrary')
    def test_linux_dl_oserror(self, mock_load_library, mock_find_library, mock_libc_ver):
        """Confirms ctype API exceptions actually propagate from JLinkarmDlInfo to call site.

        Test case:
        - initial find_library('jlinkarm') succeeds
        - the host system presents itself as GNU/Linux, and we successfully load libdl
        - but loading libdl raises OSError
        """

        mock_find_library.side_effect = [
            # find_library('jlinkarm')
            'libjlinkarm.so.6',
            # find_library('dl')
            'libdl.so.2'
        ]
        mock_load_library.side_effect = [
            # load JLink DLL
            mock.Mock(),
            # load libdl
            OSError()
        ]

        with self.assertRaises(OSError):
            lib = library.Library()
            lib.unload = mock.Mock()

        mock_find_library.assert_any_call(library.Library.JLINK_SDK_OBJECT)
        mock_find_library.assert_any_call('dl')
        self.assertEqual(2, mock_find_library.call_count)
        self.assertEqual(2, mock_load_library.call_count)

    @mock.patch('ctypes.cdll.LoadLibrary')
    def test_dll_version_linux(self, _mock_load_library):
        path = "/opt/SEGGER/JLink_Linux_V684b_x86_64/libjlinkarm.so"
        lib = library.Library(path)
        self.assertEqual(lib.dll_version(), 6.84)

    @mock.patch('ctypes.cdll.LoadLibrary')
    def test_dll_version_mac(self, _mock_load_library):
        path = "/Applications/SEGGER/JLink_V796n/libjlinkarm.dylib"
        lib = library.Library(path)
        self.assertEqual(lib.dll_version(), 7.96)


if __name__ == '__main__':
    unittest.main()



# Copyright 2019 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import behave
import pylink

import time


@behave.when('I enable RTT with config block {block_address}')
def step_enable_rtt(context, block_address):
    """Enables RTT.

    Args:
      context (Context): the ``Context`` instance
      block_address (str): the RTT config block address

    Returns:
      ``None``
    """
    jlink = context.jlink
    jlink.rtt_start(int(block_address, 16))
    while True:
        try:
            jlink.rtt_get_num_up_buffers()
            jlink.rtt_get_num_down_buffers()
            break
        except pylink.errors.JLinkRTTException as e:
            time.sleep(0.1)

    assert jlink.rtt_get_num_up_buffers() > 0
    assert jlink.rtt_get_num_down_buffers() > 0


@behave.when('I write "{message}" to RTT channel {channel}')
def step_write_rtt_channel(context, message, channel):
    """Writes a message to an RTT channel.

    Args:
      context (Context): the ``Context`` instance
      message (str): the message to write
      channel (str): the RTT channel to write to

    Returns:
      ``None``
    """
    bytes_to_write = list(bytearray(message, 'utf-8') + b'\x0A\x00')
    jlink = context.jlink
    bytes_written = jlink.rtt_write(int(channel), bytes_to_write)
    assert len(bytes_to_write) == bytes_written


@behave.then('I should read "{message}" on RTT channel {channel}')
def step_read_rtt_channel(context, message, channel):
    """Checks that the expected message is read over the RTT channel.

    Args:
      context (Context): the ``Context`` instance
      message (str): the expected message
      channel (str): the RTT channel to read from

    Returns:
      ``None``
    """
    jlink = context.jlink
    expected = str(message)

    bytes_read = []
    while len(bytes_read) < len(message):
        bytes_read += jlink.rtt_read(int(channel), 1024)
        time.sleep(0.1)

    actual = ''.join(map(chr, bytes_read)).strip()
    assert expected == actual



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from . import enums

import ctypes


class JLinkConnectInfo(ctypes.Structure):
    """J-Link connection info structure.

    Attributes:
      SerialNumber: J-Link serial number.
      Connection: type of connection (e.g. ``enums.JLinkHost.USB``)
      USBAddr: USB address if connected via USB.
      aIPAddr: IP address if connected via IP.
      Time: Time period (ms) after which UDP discover answer was received.
      Time_us: Time period (uS) after which UDP discover answer was received.
      HWVersion: Hardware version of J-Link, if connected via IP.
      abMACAddr: MAC Address, if connected via IP.
      acProduct: Product name, if connected via IP.
      acNickname: Nickname, if connected via IP.
      acFWString: Firmware string, if connected via IP.
      IsDHCPAssignedIP: Is IP address reception via DHCP.
      IsDHCPAssignedIPIsValid: True if connected via IP.
      NumIPConnections: Number of IP connections currently established.
      NumIPConnectionsIsValid: True if connected via IP.
      aPadding: Bytes reserved for future use.
    """
    _fields_ = [
        ('SerialNumber', ctypes.c_uint32),
        ('Connection', ctypes.c_ubyte),
        ('USBAddr', ctypes.c_uint32),
        ('aIPAddr', ctypes.c_uint8 * 16),
        ('Time', ctypes.c_int),
        ('Time_us', ctypes.c_uint64),
        ('HWVersion', ctypes.c_uint32),
        ('abMACAddr', ctypes.c_uint8 * 6),
        ('acProduct', ctypes.c_char * 32),
        ('acNickname', ctypes.c_char * 32),
        ('acFWString', ctypes.c_char * 112),
        ('IsDHCPAssignedIP', ctypes.c_char),
        ('IsDHCPAssignedIPIsValid', ctypes.c_char),
        ('NumIPConnections', ctypes.c_char),
        ('NumIPConnectionsIsValid', ctypes.c_char),
        ('aPadding', ctypes.c_uint8 * 34)
    ]

    def __repr__(self):
        """Returns a representation of this class.

        Args:
          self (JLinkConnectInfo): the ``JlinkConnectInfo`` instance

        Returns:
          String representation of the class.
        """
        return 'JLinkConnectInfo(%s)' % self.__str__()

    def __str__(self):
        """Returns a string representation of the connection info.

        Args:
          self (JLinkConnectInfo): the ``JLinkConnectInfo`` instance

        Returns:
          String specifying the product, its serial number, and the type of
          connection that it has (one of USB or IP).
        """
        conn = 'USB' if self.Connection == 1 else 'IP'
        return '%s <Serial No. %s, Conn. %s>' % (self.acProduct.decode(), self.SerialNumber, conn)


class JLinkFlashArea(ctypes.Structure):
    """Definition for a region of Flash.

    Attributes:
      Addr: address where the flash area starts.
      Size: size of the flash area.
    """

    _fields_ = [
        ('Addr', ctypes.c_uint32),
        ('Size', ctypes.c_uint32)
    ]

    def __repr__(self):
        """Returns a representation of the instance.

        Args:
          self (FlashArea): the ``FlashArea`` instance

        Returns:
          String representation of the Flash Area.
        """
        return '%s(%s)' % (self.__class__.__name__, self.__str__())

    def __str__(self):
        """Returns a string representation of the instance.

        Args:
          self (FlashArea): the ``FlashArea`` instance

        Returns:
          String specifying address of flash region, and its size.
        """
        return 'Address = 0x%x, Size = %s' % (self.Addr, self.Size)


class JLinkRAMArea(JLinkFlashArea):
    """Definition for a region of RAM.

    Attributes:
      Addr: address where the flash area starts.
      Size: size of the flash area.
    """
    pass


class JLinkDeviceInfo(ctypes.Structure):
    """J-Link device information.

    This structure is used to represent a device that is supported by the
    J-Link.

    Attributes:
      SizeOfStruct: Size of the struct (DO NOT CHANGE).
      sName: name of the device.
      CoreId: core identifier of the device.
      FlashAddr: base address of the internal flash of the device.
      RAMAddr: base address of the internal RAM of the device.
      EndianMode: the endian mode of the device (0 -> only little endian,
          1 -> only big endian, 2 -> both).
      FlashSize: total flash size in bytes.
      RAMSize: total RAM size in bytes.
      sManu: device manufacturer.
      aFlashArea: a list of ``JLinkFlashArea`` instances.
      aRamArea: a list of ``JLinkRAMArea`` instances.
      Core: CPU core.
    """
    _fields_ = [
        ('SizeofStruct', ctypes.c_uint32),
        ('sName', ctypes.POINTER(ctypes.c_char)),
        ('CoreId', ctypes.c_uint32),
        ('FlashAddr', ctypes.c_uint32),
        ('RAMAddr', ctypes.c_uint32),
        ('EndianMode', ctypes.c_char),
        ('FlashSize', ctypes.c_uint32),
        ('RAMSize', ctypes.c_uint32),
        ('sManu', ctypes.POINTER(ctypes.c_char)),
        ('aFlashArea', JLinkFlashArea * 32),
        ('aRAMArea', JLinkRAMArea * 32),
        ('Core', ctypes.c_uint32)
    ]

    def __init__(self, *args, **kwargs):
        """Initializes the instance.

        Populates the ``.SizeofStruct`` parameter to the size of the instance.

        Args:
          self (JLinkDeviceInfo): the ``JLinkDeviceInfo`` instance
          args: list of arguments
          kwargs: key-word arguments dictionary

        Returns:
          ``None``
        """
        super(JLinkDeviceInfo, self).__init__(*args, **kwargs)
        self.SizeofStruct = ctypes.sizeof(self)

    def __repr__(self):
        """Returns a representation of this instance.

        Args:
          self (JLinkDeviceInfo): the ``JLinkDeviceInfo`` instance

        Returns:
          Returns a string representation of the instance.
        """
        return 'JLinkDeviceInfo(%s)' % self.__str__()

    def __str__(self):
        """Returns a string representation of this instance.

        Args:
          self (JLinkDeviceInfo): the ``JLinkDeviceInfo`` instance

        Returns:
          Returns a string specifying the device name, core, and manufacturer.
        """
        manu = self.manufacturer
        return '%s <Core Id. %s, Manu. %s>' % (self.name, self.Core, manu)

    @property
    def name(self):
        """Returns the name of the device.

        Args:
          self (JLinkDeviceInfo): the ``JLinkDeviceInfo`` instance

        Returns:
          Device name.
        """
        return ctypes.cast(self.sName, ctypes.c_char_p).value.decode()

    @property
    def manufacturer(self):
        """Returns the name of the manufacturer of the device.

        Args:
          self (JLinkDeviceInfo): the ``JLinkDeviceInfo`` instance

        Returns:
          Manufacturer name.
        """
        buf = ctypes.cast(self.sManu, ctypes.c_char_p).value
        return buf.decode() if buf else None


class JLinkHardwareStatus(ctypes.Structure):
    """Definition for the hardware status information for a J-Link.

    Attributes:
      VTarget: target supply voltage.
      tck: measured state of TCK pin.
      tdi: measured state of TDI pin.
      tdo: measured state of TDO pin.
      tms: measured state of TMS pin.
      tres: measured state of TRES pin.
      trst: measured state of TRST pin.
    """
    _fields_ = [
        ('VTarget', ctypes.c_uint16),
        ('tck', ctypes.c_uint8),
        ('tdi', ctypes.c_uint8),
        ('tdo', ctypes.c_uint8),
        ('tms', ctypes.c_uint8),
        ('tres', ctypes.c_uint8),
        ('trst', ctypes.c_uint8)
    ]

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkHardwareStatus): the ``JlinkHardwareStatus`` instance

        Returns:
          String representation of the instance.
        """
        return '%s(VTarget=%dmV)' % (self.__class__.__name__, self.voltage)

    @property
    def voltage(self):
        """Returns the target supply voltage.

        This is an alias for ``.VTarget``.

        Args:
          self (JLInkHardwareStatus): the ``JLinkHardwareStatus`` instance

        Returns:
          Target supply voltage as an integer.
        """
        return self.VTarget


class JLinkGPIODescriptor(ctypes.Structure):
    """Definition for the structure that details the name and capabilities of a
    user-controllable GPIO.

    Attributes:
      acName: name of the GPIO.
      Caps: bitfield of capabilities.
    """
    _fields_ = [
        ('acName', ctypes.c_char * 32),
        ('Caps', ctypes.c_uint32)
    ]

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkGPIODescriptor): the ``JLinkGPIODescriptor`` instance

        Returns:
          String representation of the instance.
        """
        return '%s(%s)' % (self.__class__.__name__, self.__str__())

    def __str__(self):
        """Returns the GPIO name.

        Args:
          self (JLinkGPIODescriptor): the ``JLInkGPIODescriptor`` instance

        Returns:
          GPIO name.
        """
        return self.acName.decode()


class JLinkMemoryZone(ctypes.Structure):
    """Represents a CPU memory zone.

    Attributes:
      sName: initials of the memory zone.
      sDesc: name of the memory zone.
      VirtAddr: start address of the virtual address space of the memory zone.
      abDummy: reserved for future use.
    """
    _fields_ = [
        ('sName', ctypes.c_char_p),
        ('sDesc', ctypes.c_char_p),
        ('VirtAddr', ctypes.c_uint64),
        ('abDummy', (ctypes.c_uint8 * 16))
    ]

    def __repr__(self):
        """Returns a string representation of the instance

        Args:
          self: the ``JLinkMemoryZone`` instance

        Returns:
          String representation of the instance.
        """
        return '%s(%s)' % (self.__class__.__name__, self.__str__())

    def __str__(self):
        """Returns a formatted string describing the memory zone.

        Args:
          self: the ``JLinkMemoryZone`` instance

        Returns:
          String representation of the memory zone.
        """
        return '%s <Desc. %s, VirtAddr. 0x%x>' % (self.sName, self.sDesc, self.VirtAddr)

    @property
    def name(self):
        """Alias for the memory zone name.

        Args:
          self (JLinkMemoryZone): the ``JLinkMemoryZone`` instance

        Returns:
          The memory zone name.
        """
        return self.sName


class JLinkSpeedInfo(ctypes.Structure):
    """Represents information about an emulator's supported speeds.

    The emulator can support all target interface speeds calculated by dividing
    the base frequency by atleast ``MinDiv``.

    Attributes:
      SizeOfStruct: the size of this structure.
      BaseFreq: Base frequency (in HZ) used to calculate supported speeds.
      MinDiv: minimum divider allowed to divide the base frequency.
      SupportAdaptive: ``1`` if emulator supports adaptive clocking, otherwise
          ``0``.
    """
    _fields_ = [
        ('SizeOfStruct', ctypes.c_uint32),
        ('BaseFreq', ctypes.c_uint32),
        ('MinDiv', ctypes.c_uint16),
        ('SupportAdaptive', ctypes.c_uint16)
    ]

    def __init__(self):
        """Initializes the ``JLinkSpeedInfo`` instance.

        Sets the size of the structure.

        Args:
          self (JLinkSpeedInfo): the ``JLinkSpeedInfo`` instance

        Returns:
          ``None``
        """
        super(JLinkSpeedInfo, self).__init__()
        self.SizeOfStruct = ctypes.sizeof(self)

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkSpeedInfo): the ``JLinkSpeedInfo`` instance

        Returns:
          String representation of the instance.
        """
        return self.__str__()

    def __str__(self):
        """Returns this instance formatted as a string.

        Args:
          self (JLinkSpeedInfo): the ``JLinkSpeedInfo`` instance

        Returns:
          String formatted instance.
        """
        return '%s(Freq=%sHz)' % (self.__class__.__name__, self.BaseFreq)


class JLinkSWOStartInfo(ctypes.Structure):
    """Represents configuration information for collecting Serial Wire Output
    (SWO) information.

    Attributes:
      SizeofStruct: size of the structure.
      Interface: the interface type used for SWO.
      Speed: the frequency used for SWO communication in Hz.

    Note:
      You should *never* change ``.SizeofStruct`` or ``.Interface``.
    """
    _fields_ = [
        ('SizeofStruct', ctypes.c_uint32),
        ('Interface', ctypes.c_uint32),
        ('Speed', ctypes.c_uint32)
    ]

    def __init__(self):
        """Initializes the SWO start information.

        Args:
          self (JLinkSWOStartInfo): the ``JLinkSWOStartInfo`` instance

        Returns:
          ``None``
        """
        super(JLinkSWOStartInfo, self).__init__()
        self.SizeofStruct = ctypes.sizeof(self)
        self.Interface = enums.JLinkSWOInterfaces.UART

    def __repr__(self):
        """Returns a representation of this instance.

        Args:
          self (JLinkSWOStartInfo): the ``JLinkSWOStartInfo`` instance

        Returns:
          The string representation of this instance.
        """
        return self.__str__()

    def __str__(self):
        """Returns a string representation of this instance.

        Args:
          self (JLinkSWOStartInfo): the ``JLinkSWOStartInfo`` instance

        Returns:
          The string representation of this instance.
        """
        return '%s(Speed=%sHz)' % (self.__class__.__name__, self.Speed)


class JLinkSWOSpeedInfo(ctypes.Structure):
    """Structure representing information about target's supported SWO speeds.

    To calculate the supported SWO speeds, the base frequency is taken and
    divide by a number in the range of ``[ MinDiv, MaxDiv ]``.

    Attributes:
      SizeofStruct: size of the structure.
      Interface: interface type for the speed information.
      BaseFreq: base frequency (Hz) used to calculate supported SWO speeds.
      MinDiv: minimum divider allowed to divide the base frequency.
      MaxDiv: maximum divider allowed to divide the base frequency.
      MinPrescale: minimum prescaler allowed to adjust the base frequency.
      MaxPrescale: maximum prescaler allowed to adjust the base frequency.

    Note:
      You should *never* change ``.SizeofStruct`` or ``.Interface``.
    """
    _fields_ = [
        ('SizeofStruct', ctypes.c_uint32),
        ('Interface', ctypes.c_uint32),
        ('BaseFreq', ctypes.c_uint32),
        ('MinDiv', ctypes.c_uint32),
        ('MaxDiv', ctypes.c_uint32),
        ('MinPrescale', ctypes.c_uint32),
        ('MaxPrescale', ctypes.c_uint32)
    ]

    def __init__(self):
        """Initializes the J-Link SWO Speed Information instance.

        Args:
          self (JLinkSWOSpeedInfo): the ``JLinkSWOSpeedInfo`` instance

        Returns:
          ``None``
        """
        super(JLinkSWOSpeedInfo, self).__init__()
        self.SizeofStruct = ctypes.sizeof(self)
        self.Interface = enums.JLinkSWOInterfaces.UART

    def __repr__(self):
        """Returns a representation of the instance.

        Args:
          self (JLinkSWOSpeedInfo): the ``JLinkSWOSpeedInfo`` instance

        Returns:
          ``None``
        """
        return self.__str__()

    def __str__(self):
        """Returns a string representaton of the instance.

        Args:
          self (JLinkSWOSpeedInfo): the ``JLinkSWOSpeedInfo`` instance

        Returns:
          ``None``
        """
        return '%s(Interface=UART, Freq=%sHz)' % (self.__class__.__name__, self.BaseFreq)


class JLinkMOEInfo(ctypes.Structure):
    """Structure representing the Method of Debug Entry (MOE).

    The method of debug entry is a reason for which a CPU has stopped.  At any
    given time, there may be multiple methods of debug entry.

    Attributes:
      HaltReason: reason why the CPU stopped.
      Index: if cause of CPU stop was a code/data breakpoint, this identifies
        the index of the code/data breakpoint unit which causes the CPU to
        stop, otherwise it is ``-1``.
    """
    _fields_ = [
        ('HaltReason', ctypes.c_uint32),
        ('Index', ctypes.c_int)
    ]

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkMOEInfo): the ``JLinkMOEInfo`` instance

        Returns:
          A string representation of the instance.
        """
        return '%s(%s)' % (self.__class__.__name__, self.__str__())

    def __str__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkMOEInfo): the ``JLinkMOEInfo`` instance

        Returns:
          A string representation of the instance.
        """
        d = enums.JLinkHaltReasons.__dict__
        s = next(k for k, v in d.items() if v == self.HaltReason)
        if self.dbgrq():
            return s
        return s.replace('_', ' ').title()

    def dbgrq(self):
        """Returns whether this a DBGRQ.

        Args:
          self (JLinkMOEInfo): the ``JLinkMOEInfo`` instance

        Returns:
          ``True`` if this is a DBGRQ, otherwise ``False``.
        """
        return self.HaltReason == enums.JLinkHaltReasons.DBGRQ

    def code_breakpoint(self):
        """Returns whether this a code breakpoint.

        Args:
          self (JLinkMOEInfo): the ``JLinkMOEInfo`` instance

        Returns:
          ``True`` if this is a code breakpoint, otherwise ``False``.
        """
        return self.HaltReason == enums.JLinkHaltReasons.CODE_BREAKPOINT

    def data_breakpoint(self):
        """Returns whether this a data breakpoint.

        Args:
          self (JLinkMOEInfo): the ``JLinkMOEInfo`` instance

        Returns:
          ``True`` if this is a data breakpoint, otherwise ``False``.
        """
        return self.HaltReason == enums.JLinkHaltReasons.DATA_BREAKPOINT

    def vector_catch(self):
        """Returns whether this a vector catch.

        Args:
          self (JLinkMOEInfo): the ``JLinkMOEInfo`` instance

        Returns:
          ``True`` if this is a vector catch, otherwise ``False``.
        """
        return self.HaltReason == enums.JLinkHaltReasons.VECTOR_CATCH


class JLinkBreakpointInfo(ctypes.Structure):
    """Class representing information about a breakpoint.

    Attributes:
      SizeOfStruct: the size of the structure (this should not be modified).
      Handle: breakpoint handle.
      Addr: address of where the breakpoint has been set.
      Type: type flags which were specified when the breakpoint was created.
      ImpFlags: describes the current state of the breakpoint.
      UseCnt: describes how often the breakpoint is set at the same address.
    """
    _fields_ = [
        ('SizeOfStruct', ctypes.c_uint32),
        ('Handle', ctypes.c_uint32),
        ('Addr', ctypes.c_uint32),
        ('Type', ctypes.c_uint32),
        ('ImpFlags', ctypes.c_uint32),
        ('UseCnt', ctypes.c_uint32)
    ]

    def __init__(self):
        """Initializes the ``JLinkBreakpointInfo`` instance.

        Sets the size of the structure.

        Args:
          self (JLinkBreakpointInfo): the ``JLinkBreakpointInfo`` instnace

        Returns:
          ``None``
        """
        super(JLinkBreakpointInfo, self).__init__()
        self.SizeOfStruct = ctypes.sizeof(self)

    def __repr__(self):
        """Returns a formatted string describing the breakpoint.

        Args:
          self (JLinkBreakpointInfo): the ``JLinkBreakpointInfo`` instance

        Returns:
          Stirng representation of the breakpoint.
        """
        return self.__str__()

    def __str__(self):
        """Returns a formatted string describing the breakpoint.

        Args:
          self (JLinkBreakpointInfo): the ``JLinkBreakpointInfo`` instance

        Returns:
          Stirng representation of the breakpoint.
        """
        name = self.__class__.__name__
        return '%s(Handle %d, Address %d)' % (name, self.Handle, self.Addr)

    def software_breakpoint(self):
        """Returns whether this is a software breakpoint.

        Args:
          self (JLinkBreakpointInfo): the ``JLinkBreakpointInfo`` instance

        Returns:
          ``True`` if the breakpoint is a software breakpoint, otherwise
          ``False``.
        """
        software_types = [
            enums.JLinkBreakpoint.SW_RAM,
            enums.JLinkBreakpoint.SW_FLASH,
            enums.JLinkBreakpoint.SW
        ]
        return any(self.Type & stype for stype in software_types)

    def hardware_breakpoint(self):
        """Returns whether this is a hardware breakpoint.

        Args:
          self (JLinkBreakpointInfo): the ``JLinkBreakpointInfo`` instance

        Returns:
          ``True`` if the breakpoint is a hardware breakpoint, otherwise
          ``False``.
        """
        return self.Type & enums.JLinkBreakpoint.HW

    def pending(self):
        """Returns if this breakpoint is pending.

        Args:
          self (JLinkBreakpointInfo): the ``JLinkBreakpointInfo`` instance

        Returns:
          ``True`` if the breakpoint is still pending, otherwise ``False``.
        """
        return self.ImpFlags & enums.JLinkBreakpointImplementation.PENDING


class JLinkDataEvent(ctypes.Structure):
    """Class representing a data event.

    A data may halt the CPU, trigger SWO output, or trigger trace output.

    Attributes:
      SizeOfStruct: the size of the structure (this should not be modified).
      Type: the type of the data event (this should not be modified).
      Addr: the address on which the watchpoint was set
      AddrMask: the address mask used for comparision.
      Data: the data on which the watchpoint has been set.
      DataMask: the data mask used for comparision.
      Access: the control data on which the event has been set.
      AccessMask: the control mask used for comparison.
    """
    _fields_ = [
        ('SizeOfStruct', ctypes.c_int),
        ('Type', ctypes.c_int),
        ('Addr', ctypes.c_uint32),
        ('AddrMask', ctypes.c_uint32),
        ('Data', ctypes.c_uint32),
        ('DataMask', ctypes.c_uint32),
        ('Access', ctypes.c_uint8),
        ('AccessMask', ctypes.c_uint8)
    ]

    def __init__(self):
        """Initializes the ``JLinkDataEvent`` instance.

        Sets the size of the structure.

        Args:
          self (JLinkDataEvent): the ``JLinkDataEvent`` instance

        Returns:
          ``None``
        """
        super(JLinkDataEvent, self).__init__()
        self.SizeOfStruct = ctypes.sizeof(self)
        self.Type = enums.JLinkEventTypes.BREAKPOINT

    def __repr__(self):
        """Returns a string representation of the data event.

        Args:
          self (JLinkDataEvent): the ``JLinkDataEvent`` instance

        Returns:
          A string representation of the data event.
        """
        return self.__str__()

    def __str__(self):
        """Returns a string representation of the data event.

        Args:
          self (JLinkDataEvent): the ``JLinkDataEvent`` instance

        Returns:
          A string representation of the data event.
        """
        name = self.__class__.__name__
        return '%s(Type %d, Address %d)' % (name, self.Type, self.Addr)


class JLinkWatchpointInfo(ctypes.Structure):
    """Class representing information about a watchpoint.

    Attributes:
      SizeOfStruct: the size of the structure (this should not be modified).
      Handle: the watchpoint handle.
      Addr: the address the watchpoint was set at.
      AddrMask: the address mask used for comparison.
      Data: the data on which the watchpoint was set.
      DataMask: the data mask used for comparision.
      Ctrl: the control data on which the breakpoint was set.
      CtrlMask: the control mask used for comparison.
      WPUnit: the index of the watchpoint unit.
    """
    _fields_ = [
        ('SizeOfStruct', ctypes.c_uint32),
        ('Handle', ctypes.c_uint32),
        ('Addr', ctypes.c_uint32),
        ('AddrMask', ctypes.c_uint32),
        ('Data', ctypes.c_uint32),
        ('DataMask', ctypes.c_uint32),
        ('Ctrl', ctypes.c_uint32),
        ('CtrlMask', ctypes.c_uint32),
        ('WPUnit', ctypes.c_uint8)
    ]

    def __init__(self):
        """Initializes the ``JLinkWatchpointInfo`` instance.

        Sets the size of the structure.

        Args:
          self (JLinkWatchpointInfo): the ``JLinkWatchpointInfo`` instance

        Returns:
          ``None``
        """
        super(JLinkWatchpointInfo, self).__init__()
        self.SizeOfStruct = ctypes.sizeof(self)

    def __repr__(self):
        """Returns a formatted string describing the watchpoint.

        Args:
          self (JLinkWatchpointInfo): the ``JLinkWatchpointInfo`` instance

        Returns:
          String representation of the watchpoint.
        """
        return self.__str__()

    def __str__(self):
        """Returns a formatted string describing the watchpoint.

        Args:
          self (JLinkWatchpointInfo): the ``JLinkWatchpointInfo`` instance

        Returns:
          String representation of the watchpoint.
        """
        name = self.__class__.__name__
        return '%s(Handle %d, Address %d)' % (name, self.Handle, self.Addr)


class JLinkStraceEventInfo(ctypes.Structure):
    """Class representing the STRACE event information.

    Attributes:
      SizeOfStruct: size of the structure.
      Type: type of event.
      Op: the STRACE operation to perform.
      AccessSize: access width for trace events.
      Reserved0: reserved.
      Addr: specifies the load/store address for data.
      Data: the data to be compared for the operation for data access events.
      DataMask: bitmask for bits of data to omit in comparision for data access
        events.
      AddrRangeSize: address range for range events.
    """
    _fields_ = [
        ('SizeOfStruct', ctypes.c_uint32),
        ('Type', ctypes.c_uint8),
        ('Op', ctypes.c_uint8),
        ('AccessSize', ctypes.c_uint8),
        ('Reserved0', ctypes.c_uint8),
        ('Addr', ctypes.c_uint64),
        ('Data', ctypes.c_uint64),
        ('DataMask', ctypes.c_uint64),
        ('AddrRangeSize', ctypes.c_uint32)
    ]

    def __init__(self):
        """Initializes the ``JLinkStraceEventInfo`` instance.

        Sets the size of the structure.

        Args:
          self (JLinkStraceEventInfo): the ``JLinkStraceEventInfo`` instance

        Returns:
          ``None``
        """
        super(JLinkStraceEventInfo, self).__init__()
        self.SizeOfStruct = ctypes.sizeof(self)

    def __repr__(self):
        """Returns a formatted string describing the event info.

        Args:
          self (JLinkStraceEventInfo): the ``JLinkStraceEventInfo`` instance

        Returns:
          String representation of the event info.
        """
        return self.__str__()

    def __str__(self):
        """Returns a formatted string describing the event info.

        Args:
          self (JLinkStraceEventInfo): the ``JLinkStraceEventInfo`` instance

        Returns:
          String representation of the event information.
        """
        name = self.__class__.__name__
        return '%s(Type=%d, Op=%d)' % (name, self.Type, self.Op)


class JLinkTraceData(ctypes.Structure):
    """Structure representing trace data returned by the trace buffer.

    Attributes:
      PipeStat: type of trace data.
      Sync: sync point in buffer.
      Packet: trace data packet.
    """
    _fields_ = [
        ('PipeStat', ctypes.c_uint8),
        ('Sync', ctypes.c_uint8),
        ('Packet', ctypes.c_uint16)
    ]

    def __repr__(self):
        """Returns a string representation of the trace data instance.

        Args:
          self (JLinkTraceData): the ``JLinkTraceData`` instance.

        Returns:
          A string representation of the instance.
        """
        return self.__str__()

    def __str__(self):
        """Returns a string representation of the trace data instance.

        Args:
          self (JLinkTraceData): the ``JLinkTraceData`` instance.

        Returns:
          A string representation of the instance.
        """
        return '%s(%d)' % (self.__class__.__name__, self.Packet)

    def instruction(self):
        """Returns whether the data corresponds to an executed instruction.

        Args:
          self (JLinkTraceData): the ``JLinkTraceData`` instance.

        Returns:
          ``True`` if this is trace data for an executed instruction.
        """
        return (self.PipeStat == 0)

    def data_instruction(self):
        """Returns whether the data corresponds to an data instruction.

        Args:
          self (JLinkTraceData): the ``JLinkTraceData`` instance.

        Returns:
          ``True`` if this is trace data for an data instruction.
        """
        return (self.PipeStat == 1)

    def non_instruction(self):
        """Returns whether the data corresponds to an un-executed instruction.

        Args:
          self (JLinkTraceData): the ``JLinkTraceData`` instance.

        Returns:
          ``True`` if this is trace data for an un-executed instruction.
        """
        return (self.PipeStat == 2)

    def wait(self):
        """Returns whether the data corresponds to a wait.

        Args:
          self (JLinkTraceData): the ``JLinkTraceData`` instance.

        Returns:
          ``True`` if this is trace data for a wait.
        """
        return (self.PipeStat == 3)

    def branch(self):
        """Returns whether the data corresponds to a branch execution.

        Args:
          self (JLinkTraceData): the ``JLinkTraceData`` instance.

        Returns:
          ``True`` if this is trace data for a branch execution.
        """
        return (self.PipeStat == 4)

    def data_branch(self):
        """Returns whether the data corresponds to a branch with data.

        Args:
          self (JLinkTraceData): the ``JLinkTraceData`` instance.

        Returns:
          ``True`` if this is trace data for a branch with data.
        """
        return (self.PipeStat == 5)

    def trigger(self):
        """Returns whether the data corresponds to a trigger event.

        Args:
          self (JLinkTraceData): the ``JLinkTraceData`` instance.

        Returns:
          ``True`` if this is trace data for a trigger event.
        """
        return (self.PipeStat == 6)

    def trace_disabled(self):
        """Returns whether the data corresponds to trace being disabled.

        Args:
          self (JLinkTraceData): the ``JLinkTraceData`` instance.

        Returns:
          ``True`` if this is trace data for the trace disabled event.
        """
        return (self.PipeStat == 7)


class JLinkTraceRegion(ctypes.Structure):
    """Structure describing a trace region.

    Attributes:
      SizeOfStruct: size of the structure.
      RegionIndex: index of the region.
      NumSamples: number of samples in the region.
      Off: offset in the trace buffer.
      RegionCnt: number of trace regions.
      Dummy: unused.
      Timestamp: timestamp of last event written to buffer.
    """
    _fields_ = [
        ('SizeOfStruct', ctypes.c_uint32),
        ('RegionIndex', ctypes.c_uint32),
        ('NumSamples', ctypes.c_uint32),
        ('Off', ctypes.c_uint32),
        ('RegionCnt', ctypes.c_uint32),
        ('Dummy', ctypes.c_uint32),
        ('Timestamp', ctypes.c_uint64)
    ]

    def __init__(self):
        """Initializes the trace region.

        Sets the size of the structure.

        Args:
          self (JLinkTraceRegion): the ``JLinkTraceRegion`` instance.

        Returns:
          ``None``
        """
        super(JLinkTraceRegion, self).__init__()
        self.SizeOfStruct = ctypes.sizeof(self)

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkTraceRegion): the ``JLinkTraceRegion`` instance.

        Returns:
          String representation of the trace region.
        """
        return self.__str__()

    def __str__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkTraceRegion): the ``JLinkTraceRegion`` instance.

        Returns:
          String representation of the trace region.
        """
        return '%s(Index=%d)' % (self.__class__.__name__, self.RegionIndex)


class JLinkRTTerminalStart(ctypes.Structure):
    """Structure used to configure an RTT instance.

    Attributes:
      ConfigBlockAddress: Address of the RTT block.
    """
    _fields_ = [
        ('ConfigBlockAddress', ctypes.c_uint32),
        ('Reserved', ctypes.c_uint32 * 3)
    ]

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkRTTerminalStart): RTT start instance.

        Returns:
          String representation of the instance.
        """
        return '%s(ConfigAddress=0x%X)' % (self.__class__.__name__, self.ConfigBlockAddress)

    def __str__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkRTTerminalStart): RTT start instance.

        Returns:
          String representation of the instance.
        """
        return self.__repr__()


class JLinkRTTerminalBufDesc(ctypes.Structure):
    """Structure describing a RTT buffer.

    Attributes:
      BufferIndex: index of the buffer to request information about.
      Direction: direction of the upper (`0` for up, `1` for Down).
      acName: Name of the buffer.
      SizeOfBuffer: size of the buffer in bytes.
      Flags: flags set on the buffer.
    """
    _fields_ = [
        ('BufferIndex', ctypes.c_int32),
        ('Direction', ctypes.c_uint32),
        ('acName', ctypes.c_char * 32),
        ('SizeOfBuffer', ctypes.c_uint32),
        ('Flags', ctypes.c_uint32)
    ]

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkRTTerminalBufDesc): the terminal buffer descriptor.

        Returns:
          String representation of the buffer descriptor.
        """
        return '%s(Index=%d, Name=%s)' % (self.__class__.__name__, self.BufferIndex, self.name)

    def __str__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkRTTerminalBufDesc): the terminal buffer descriptor.

        Returns:
          String representation of the buffer descriptor.
        """
        dir_string = 'up' if self.up else 'down'
        return '%s <Index=%d, Direction=%s, Size=%s>' % (self.name, self.BufferIndex,
                                                         dir_string, self.SizeOfBuffer)

    @property
    def up(self):
        """Returns a boolean indicating if the buffer is an 'UP' buffer.

        Args:
          self (JLinkRTTerminalBufDesc): the terminal buffer descriptor.

        Returns:
          ``True`` if the buffer is an 'UP' buffer, otherwise ``False``.
        """
        return (self.Direction == 0)

    @property
    def down(self):
        """Returns a boolean indicating if the buffer is an 'DOWN' buffer.

        Args:
          self (JLinkRTTerminalBufDesc): the terminal buffer descriptor.

        Returns:
          ``True`` if the buffer is an 'DOWN' buffer, otherwise ``False``.
        """
        return (self.Direction == 1)

    @property
    def name(self):
        """Returns the name of the buffer.

        Args:
          self (JLinkRTTerminalBufDesc): the terminal buffer descriptor.

        Returns:
          String name of the buffer.
        """
        return self.acName.decode()


class JLinkRTTerminalStatus(ctypes.Structure):
    """Structure describing the status of the RTT terminal.

    Attributes:
      NumBytesTransferred: number of bytes sent to the client application.
      NumBytesRead: number of bytes read from the target.
      HostOverflowCount: number of overflows on the host.
      IsRunning: if RTT is running.
      NumUpBuffers: number of 'UP' buffers.
      NumDownBuffers: number of 'DOWN' buffers.
    """
    _fields_ = [
        ('NumBytesTransferred', ctypes.c_uint32),
        ('NumBytesRead', ctypes.c_uint32),
        ('HostOverflowCount', ctypes.c_int),
        ('IsRunning', ctypes.c_int),
        ('NumUpBuffers', ctypes.c_int),
        ('NumDownBuffers', ctypes.c_int),
        ('Reserved', ctypes.c_uint32 * 2)
    ]

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkRTTerminalStatus): the status instance.

        Returns:
          Strings representation of the status.
        """
        return '%s(NumUpBuffers=%d, NumDownBuffers=%d)' % (self.__class__.__name__,
                                                           self.NumUpBuffers, self.NumDownBuffers)

    def __str__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkRTTerminalStatus): the status instance.

        Returns:
          Strings representation of the status.
        """
        return 'Status <NumUpBuffers=%d, NumDownBuffers=%d, Running=%s>' % (self.NumUpBuffers,
                                                                            self.NumDownBuffers,
                                                                            self.IsRunning)


class JLinkPowerTraceSetup(ctypes.Structure):
    """Structure used to setup the power tracing.

    Attributes:
      SizeOfStruct: Size of the struct (DO NOT CHANGE).
      ChannelMask: Bitmask indicating which channels to enable for capturing.
      SampleFreq: Sampling frequency in Hertz.
      RefSelect: Identifier of the reference value stored with every trace
        (see ``enums.JLinkPowerTraceRef``).
      EnableCond: `1` is tracing is only captured when CPU is running.
    """
    _fields_ = [
        ('SizeOfStruct', ctypes.c_int32),
        ('ChannelMask', ctypes.c_uint32),
        ('SampleFreq', ctypes.c_uint32),
        ('RefSelect', ctypes.c_int32),
        ('EnableCond', ctypes.c_int32)
    ]

    def __init__(self):
        """Initializes the ``JLinkPowerTraceSetup`` instance.

        Sets the size of the structure.

        Args:
          self (JLinkPowerTraceSetup): the power trace instance.

        Returns:
          ``None``
        """
        super(JLinkPowerTraceSetup, self).__init__()
        self.SizeOfStruct = ctypes.sizeof(self)

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkPowerTraceSetup): the ``JLinkPowerTraceSetup`` instance

        Returns:
          String representation of the instance.
        """
        return self.__str__()

    def __str__(self):
        """Returns this instance formatted as a string.

        Args:
          self (JLinkPowerTraceSetup): the ``JLinkPowerTraceSetup`` instance

        Returns:
          String formatted instance.
        """
        return '%s(Channel Mask=%s, Freq=%uHz)' % (self.__class__.__name__, bin(self.ChannelMask), self.SampleFreq)


class JLinkPowerTraceItem(ctypes.Structure):
    """Structure used to represent a stored power trace item.

    Attributes:
      RefValue: the stored reference value.
      Value: the actual recorded trace value.
    """
    _fields_ = [
        ('RefValue', ctypes.c_uint32),
        ('Value', ctypes.c_uint32)
    ]

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkPowerTraceItem): the ``JLinkPowerTraceItem`` instance

        Returns:
          String representation of the instance.
        """
        return self.__str__()

    def __str__(self):
        """Returns this instance formatted as a string.

        Args:
          self (JLinkPowerTraceItem): the ``JLinkPowerTraceItem`` instance

        Returns:
          String formatted instance.
        """
        return '%s(Value=%u, Reference=%u)' % (self.__class__.__name__, self.Value, self.RefValue)


class JLinkPowerTraceCaps(ctypes.Structure):
    """Structure used to retrieve or specify available power tracing channels.

    Attributes:
      SizeOfStruct: the size of this structure (in bytes).
      ChannelMask: bitmask of available channels.
    """
    _fields_ = [
        ('SizeOfStruct', ctypes.c_int32),
        ('ChannelMask', ctypes.c_uint32)
    ]

    def __init__(self):
        """Initializes the ``JLinkPowerTraceCaps`` instance.

        Sets the size of the structure.

        Args:
          self (JLinkPowerTraceCaps): the power trace caps instance.

        Returns:
          ``None``
        """
        super(JLinkPowerTraceCaps, self).__init__()
        self.SizeOfStruct = ctypes.sizeof(self)

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkPowerTraceCaps): the caps instance.

        Returns:
          String representation of the instance.
        """
        return self.__str__()

    def __str__(self):
        """Returns this instance formatted as a string.

        Args:
          self (JLinkPowerTraceCaps): the caps instance.

        Returns:
          String formatted instance.
        """
        return '%s(Channel Mask=%s)' % (self.__class__.__name__, bin(self.ChannelMask))


class JLinkPowerTraceChannelCaps(ctypes.Structure):
    """Structure representing the capabilities for the queried channels.

    Attributes:
      SizeOfStruct: the size of this structure (in bytes).
      BaseSampleFreq: the base sampling frequeny (in Hertz).
      MinDiv: the minimum divider of the base sampling frequency.
    """
    _fields_ = [
        ('SizeOfStruct', ctypes.c_int32),
        ('BaseSampleFreq', ctypes.c_uint32),
        ('MinDiv', ctypes.c_uint32)
    ]

    def __init__(self):
        """Initializes the ``JLinkPowerTraceChannelCaps`` instance.

        Sets the size of the structure.

        Args:
          self (JLinkPowerTraceChannelCaps): the channel capabilities.

        Returns:
          ``None``
        """
        super(JLinkPowerTraceChannelCaps, self).__init__()
        self.SizeOfStruct = ctypes.sizeof(self)

    @property
    def max_sample_freq(self):
        """Returns the maximum sample frequency that can be used.

        The maximum sampling frequency is the largest frequency that can be
        specified when configuring power tracing, and is computed based on the
        minimum divider and base sampling frequency.

        Args:
          self (JLinkPowerTraceChannelCaps): the channel capabilities.

        Returns:
          ``float``
        """
        return (self.BaseSampleFreq * 1.0) / self.MinDiv

    def __repr__(self):
        """Returns a string representation of the instance.

        Args:
          self (JLinkPowerTraceChannelCaps): the channel capabilities.

        Returns:
          String representation of the instance.
        """
        return self.__str__()

    def __str__(self):
        """Returns this instance formatted as a string.

        Args:
          self (JLinkPowerTraceChannelCaps): the channel capabilities.

        Returns:
          String formatted instance.
        """
        return '%s(SampleFreq=%uHz, MinDiv=%u)' % (self.__class__.__name__, self.BaseSampleFreq, self.MinDiv)


class JLinkJTAGDeviceInfo(ctypes.Structure):
    """Structure representing the information of a device on the JTAG scan chain.

    Attributes:
      sName: the name of the device.
      IRLen: instruction register length.
      IRPrint: instruction register print.
      DeviceId: JTAG id.
    """
    _fields_ = [
        ('sName', ctypes.c_char_p),
        ('IRLen', ctypes.c_uint32),
        ('IRPrint', ctypes.c_uint32),
        ('DeviceId', ctypes.c_uint32)
    ]

    def __repr__(self):
        """Returns a representation of this instance.

        Args:
          self (JLinkJTAGDeviceInfo): the ``JLinkJTAGDeviceInfo`` instance

        Returns:
          Returns a string representation of the instance.
        """
        return 'JLinkJTAGDeviceInfo(%s)' % self.__str__()

    def __str__(self):
        """Returns a string representation of this instance.

        Args:
          self (JLinkJTAGDeviceInfo): the ``JLinkJTAGDeviceInfo`` instance

        Returns:
          Returns a string specifying the device name and ID.
        """
        return '%s <Device Id. %s>' % (self.name, self.DeviceId)

    @property
    def name(self):
        """Returns the name of the JTAG device.

        Args:
          self (JLinkJTAGDeviceInfo): the ``JLinkJTAGDeviceInfo`` instance

        Returns:
          Device name.
        """
        return ctypes.cast(self.sName, ctypes.c_char_p).value.decode()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink.enums as enums
import pylink.util as util

import mock

try:
    import StringIO
except ImportError:
    import io as StringIO
import unittest


class TestUtil(unittest.TestCase):
    """Unit test for the `util` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestUtil): the `TestUtil` instance

        Returns:
          `None`
        """
        pass

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestUtil): the `TestUtil` instance

        Returns:
          `None`
        """
        pass

    def test_is_integer(self):
        """Tests that the `is_integer()` method returns correctly.

        Args:
          self (TestUtil): the `TestUtil` instance

        Returns:
          `None`
        """
        self.assertTrue(util.is_integer(4))
        self.assertTrue(util.is_integer(0))
        self.assertTrue(util.is_integer(-1))

        self.assertFalse(util.is_integer('4'))
        self.assertFalse(util.is_integer('Stranger Things'))

    def test_is_natural(self):
        """Tests that the `is_natural()` method returns correctly.

        Args:
          self (TestUtil): the `TestUtil` instance

        Returns:
          `None`
        """
        self.assertTrue(util.is_natural(4))
        self.assertTrue(util.is_natural(0))

        self.assertFalse(util.is_natural(-1))
        self.assertFalse(util.is_natural('4'))
        self.assertFalse(util.is_natural('The 100'))

    def test_is_os_64bit(self):
        """Tests that the ``is_os_64bit()`` method returns correctly.

        It should return ``True`` on 64-bit platforms, otherwise ``False``.

        Args:
          self (TestUtil): the ``TestUtil`` instance

        Returns:
          ``None``
        """
        with mock.patch('platform.machine') as mock_machine:
            mock_machine.return_value = 'i386'
            self.assertFalse(util.is_os_64bit())

            mock_machine.return_value = ''
            self.assertFalse(util.is_os_64bit())

            mock_machine.return_value = 'i686'
            self.assertFalse(util.is_os_64bit())

            mock_machine.return_value = 'x86_64'
            self.assertTrue(util.is_os_64bit())

    def test_noop(self):
        """Tests that the `noop()` method does nothing and takes any args.

        Args:
          self (TestUtil): the `TestUtil` instance

        Returns:
          `None`
        """
        self.assertEqual(None, util.noop())
        self.assertEqual(None, util.noop(*range(100)))
        self.assertEqual(None, util.noop(arg=4))

    def test_unsecure_hook_dialog(self):
        """Tests that the unsecure hook dialog always returns `YES`.

        Args:
          self (TestUtil): the `TestUtil` instance

        Returns:
          `None`
        """
        self.assertEqual(enums.JLinkFlags.DLG_BUTTON_NO,
                         util.unsecure_hook_dialog('', '', 0))

    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    def test_progress_bar(self, stream):
        """Tests the progress bar calls the appropriate stream functions.

        When percent is full, the `progress_bar()` should append a newline to
        the stream, otherwise not.

        Args:
          self (TestUtil): the `TestUtil` instance
          stream (StringIO): the mock output stream

        Returns:
          `None`
        """
        self.assertEqual(None, util.progress_bar(0, 100))
        self.assertFalse(stream.getvalue() == '\n')

        self.assertEqual(None, util.progress_bar(100, 100))

        messages = stream.getvalue().split('\n')
        self.assertEqual(2, len(messages))
        self.assertTrue(messages.pop(0).endswith(' '))
        self.assertTrue(messages.pop(0) == '')

    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    def test_flash_progress_callback(self, stream):
        """Tests that the callback triggers a progress bar.

        Args:
          self (TestUtil): the `TestUtil` instance
          stream (StringIO): the mock output stream

        Returns:
          `None`
        """
        self.assertEqual(None, util.flash_progress_callback('compare', '', 0))
        self.assertEqual('', stream.getvalue())

        self.assertEqual(None, util.flash_progress_callback('Erase', '', 0))
        self.assertTrue(len(stream.getvalue()) > 0)

    def test_calculate_parity(self):
        """Tests that the parity is properly calculated.

        Args:
          self (TestUtil): the `TestUtil` instance

        Returns:
          `None`
        """
        self.assertEqual(1, util.calculate_parity(1))
        self.assertEqual(1, util.calculate_parity(2))
        self.assertEqual(0, util.calculate_parity(3))

    def test_calculate_parity_invalid(self):
        """Tests that an exception is raised for invalid args to `parity()`.

        Args:
          self (TestUtil): the `TestUtil` instance

        Returns:
          `None`
        """
        with self.assertRaises(ValueError):
            util.calculate_parity('4')

        with self.assertRaises(ValueError):
            util.calculate_parity(-1)


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink.enums as enums
import pylink.protocols.swd as swd
import pylink.unlockers as unlock

import mock

import unittest


class TestUnlockKinetis(unittest.TestCase):
    """Tests the `unlock_kinetis` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestUnlockKinetis): the `TestUnlockKinetis` instance

        Returns:
          `None`
        """
        assertRaisesRegexp = getattr(self, 'assertRaisesRegexp', None)
        self.assertRaisesRegexp = getattr(self, 'assertRaisesRegex', assertRaisesRegexp)

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestUnlockKinetis): the `TestUnlockKinetis` instance

        Returns:
          `None`
        """
        pass

    def test_unlock_kinetis_disconnected(self):
        """Tests unlock Kinetis on a disconnected J-Link.

        This should raise an error.

        Args:
          self (TestUnlockKinetis): the `TestUnlockKinetis` instance

        Returns:
          `None`
        """
        mock_jlink = mock.Mock()
        mock_jlink.connected.return_value = False

        with self.assertRaises(ValueError):
            unlock.unlock_kinetis(mock_jlink)

    def test_unlock_kinetis_unsupported_tif(self):
        """Tests trying to unlock Kinetis with an unsupported TIF.

        Args:
          self (TestUnlockKinetis): the `TestUnlockKinetis` instance

        Returns:
          `None`
        """
        mock_jlink = mock.Mock()

        with self.assertRaises(NotImplementedError):
            mock_jlink.tif = enums.JLinkInterfaces.FINE
            unlock.unlock_kinetis(mock_jlink)

        with self.assertRaises(NotImplementedError):
            mock_jlink.tif = enums.JLinkInterfaces.ICSP
            unlock.unlock_kinetis(mock_jlink)

        with self.assertRaises(NotImplementedError):
            mock_jlink.tif = enums.JLinkInterfaces.SPI
            unlock.unlock_kinetis(mock_jlink)

        with self.assertRaises(NotImplementedError):
            mock_jlink.tif = enums.JLinkInterfaces.C2
            unlock.unlock_kinetis(mock_jlink)

    def test_unlock_kinetis_jtag(self):
        """Tests unlock Kinetis on JTAG.

        Args:
          self (TestUnlockKinetis): the `TestUnlockKinetis` instance

        Returns:
          `None`
        """
        mock_jlink = mock.Mock()
        mock_jlink.tif = enums.JLinkInterfaces.JTAG

        with self.assertRaisesRegexp(NotImplementedError, 'JTAG'):
            unlock.unlock_kinetis(mock_jlink)

    @mock.patch('time.sleep')
    def test_unlock_kinetis_identify_failed(self, mock_sleep):
        """Tests that unlock Kinetis fails if device fails to identify.

        Args:
          self (TestUnlockKinetis): the `TestUnlockKinetis` instance
          mock_sleep (Mock): mocked `time.sleep()` function

        Returns:
          `None`
        """
        mock_jlink = mock.Mock()
        mock_jlink.tif = enums.JLinkInterfaces.SWD
        for flags in [0, (0x2 << 28), ((0x2 << 28) | (0xBA01 << 12))]:
            mock_jlink.coresight_read.return_value = flags
            self.assertFalse(unlock.unlock_kinetis(mock_jlink))

    @mock.patch('time.sleep')
    def test_unlock_kinetis_read_fail(self, mock_sleep):
        """Tests unlock Kinetis failing during a read.

        At any point, the unlock can fail if it tries to read from the an
        MDM-AP register and a fault occurs.

        Args:
          self (TestUnlockKinetis): the `TestUnlockKinetis` instance
          mock_sleep (Mock): mocked `time.sleep()` function

        Returns:
          `None`
        """
        mock_jlink = mock.Mock()
        mock_jlink.tif = enums.JLinkInterfaces.SWD

        flags = (0x2 << 28) | (0xBA01 << 12) | 1
        mock_jlink.coresight_read.side_effect = [flags, 0xFF]

        mock_jlink.swd_write.return_value = 0   # ACK
        mock_jlink.swd_read8.return_value = -1  # Invalid Read
        mock_jlink.swd_read32.return_value = 2  # Data

        res = unlock.unlock_kinetis(mock_jlink)
        self.assertFalse(res)

        self.assertEqual(1, mock_jlink.set_reset_pin_low.call_count)
        self.assertEqual(1, mock_jlink.set_reset_pin_high.call_count)
        self.assertEqual(0, mock_jlink.reset.call_count)

    @mock.patch('time.sleep')
    def test_unlock_kinetis_success(self, mock_sleep):
        """Tests unlock Kinetis succesfully unlocking the device.

        Args:
          self (TestUnlockKinetis): the `TestUnlockKinetis` instance
          mock_sleep (Mock): mocked `time.sleep()` function

        Returns:
          `None`
        """
        mock_jlink = mock.Mock()
        mock_jlink.tif = enums.JLinkInterfaces.SWD

        flags = (0x2 << 28) | (0xBA01 << 12) | 1
        mock_jlink.coresight_read.side_effect = [flags, 0xFF]

        ack = swd.Response.STATUS_ACK
        wait = swd.Response.STATUS_WAIT

        mocked_data = mock.Mock()
        mocked_data.read_data = [
            0x0,  # Wait
            0x1,  # Ack
            0x3,  # Flash Ready Bit Set
            0x1,  # Ack
            0x3,  # Erase Command Bit Set
            0x1,  # Ack
            0x0,  # Flash Erase In Progress Bit Cleared
        ]

        mocked_data.status_and_parity_data = [
            ack,   # Error clearing write request.
            ack,   # Selecting the MDM-AP Status Register
            wait,  # First poll, wait
            ack,   # Second poll, continue
            0x1,   # Parity
            ack,   # Flash Ready ACK
            0x0,   # Flash Ready Parity
            0x1,   # Mass Erase Request
            ack,   # Erase Command ACK
            0x1,   # Erase Command Parity
            ack,   # Erase Command ACK
            0x0,   # Erase Command Parity
            ack,   # Flash Erase Command ACK
            0x1,   # Flash Erase Command Parity
            ack,   # Flash Erase Command ACK
            0x0,   # Flash Erase Command Parity
        ]

        # ACK
        mock_jlink.swd_write.return_value = 0

        # Status and Parity
        mock_jlink.swd_read8.side_effect = mocked_data.status_and_parity_data

        # Data
        mock_jlink.swd_read32.side_effect = mocked_data.read_data

        res = unlock.unlock_kinetis(mock_jlink)
        self.assertTrue(res)

        self.assertEqual(1, mock_jlink.reset.call_count)
        self.assertEqual(1, mock_jlink.set_reset_pin_low.call_count)
        self.assertEqual(1, mock_jlink.set_reset_pin_high.call_count)


if __name__ == '__main__':
    unittest.main()
